# calcFields.tcl

#  sumRMax(no)      gtab: 
# Domain -- US EU JP CH RU XX; CpT --  "  1 "; 
# Usage -- RnD Unknown Industry Gov.Mil; logRMax --  RMax
# pcRMax --  RMax   RMax    
# SEGMENT --    Segment
# MNFCTR --    Manufacturer

proc calculations { } {
   global gtab max_no sumRMax 

   log "========================"
   log "Calculations"
   
   #    
   for {set no 0} {$no <= $max_no} {incr no} {
      #  Linpack-   
      set s 0.0
      for {set i 1} {$i <= 500} {incr i} {
         set s [expr double($gtab($no,$i,RMax)) + $s]
      }
      set sumRMax($no) $s
   
      for {set i 1} {$i <= 500} {incr i} {
         #  : US, EU, JP, CH, RU, XX
         set gtab($no,$i,Domain) [domain $gtab($no,$i,Country)]
         #   1 
         set gtab($no,$i,CpT) [cpt $gtab($no,$i,RPeak) $gtab($no,$i,Cores) \
                                   $gtab($no,$i,Proc.Frequency)]
         #  
         set gtab($no,$i,Usage) [usage $gtab($no,$i,Segment) \
                                       $gtab($no,$i,ApplicationArea)]
         #   
         set gtab($no,$i,SEGMENT) [segment $gtab($no,$i,Segment) $no $i]
         #  
         set gtab($no,$i,MNFCTR)  [manufacturer $gtab($no,$i,Manufacturer)]
         # 
         set a ""
         set b ""
         set c ""
         catch {set a $gtab($no,$i,InterconnectFamily)}
         catch {set b $gtab($no,$i,Interconnect)}
         catch {set c $gtab($no,$i,Computer)}
         set gtab($no,$i,IC)  [interconnect $a $b $c]
         # 
         set a ""
         set b ""
         set c ""
         set d ""
         catch {set a $gtab($no,$i,Processor)}
         catch {set b $gtab($no,$i,ProcessorFamily)}
         catch {set c $gtab($no,$i,ProcessorGeneration)}
         catch {set d $gtab($no,$i,ProcessorTechnology)}
         set gtab($no,$i,CHIP)  [chip $a $b $c $d]
         #  RMax
         set gtab($no,$i,logRMax) [expr log($gtab($no,$i,RMax))]
         #   sumRMax
         set gtab($no,$i,pcRMax) [expr double($gtab($no,$i,RMax))/$s]
      }
   }
}

proc chip { p pf pg pt } {
   switch -glob "$p $pf $pg $pt" {
      *Alpha*    { return  Alpha       }
      *AMD*      { return  AMD         }
      *IBM*      { return  IBM         }
      *Intel*    { return  Intel       }
      *MIPS*     { return  MIPS        }
      *PA-RISC*  { return  HP          }
      *SPARC*    { return  SPARC       }
      *Spark*    { return  SPARC       }
      *Itanium*  { return  Intel       }
      *Opteron*  { return  AMD         }
      *POWER*    { return  IBM         }
      *Power*    { return  IBM         }
      *Xeon*     { return  Intel       }
      *BQC*      { return  IBM         }
      *Core*     { return  Intel       }
      default    { return  Proprietary }
}  }

proc interconnect { icf ic comp } {
   switch -glob "$icf $ic" {
      *Cray*        { return Custom     }
      *Custom*      { return Custom     }
      *Proprietary* { return Custom     }
      *NUMAlink*    { return Custom     }
      *Quadrics*    { return Quadrics   }
      *Infiniband*  { return Infiniband }
      *Myrinet*     { return Myrinet    }
      *SCI*         { return SCI        }
      *Ethernet*    { return Ethernet   }
      default       { return Custom     }
}  }

proc manufacturer { m } {
   switch -glob $m {
      *IBM*      { return IBM }
      *Cray*     { return Cray }
      *Hewlett*  { return HP } 
      default    { return Other }
   }

#      *SGI*      { return SGI } 
#      *Bull*     { return Bull }
#      *Fujitsu*  { return Fujitsu } 
#   *Appro*    { return Appro }
#   *Dell*     { return Dell }
#   *Hitachi*  { return Hitachi }
#   *Sun*      { return Sun } 

}


proc segment { seg no i } {
   switch $seg {
      Government -
      Vendor     -
      Industry   -
      Classified -
      Research   -
      Academic   { return $seg }
      default    { log "Error!! no=$no, i=$i, Segment=$seg"
                   return Classified
                 }
   }
}

#   1 
proc cpt { peak cores frec } {
   #        (  )
   set ok 0
   catch {
      set x [expr double($peak) / double($cores) * 1000 / double($frec) ]
      set ok 1
   }  rc
   if {$ok} { return $x }
   # 
   return ""
}

#  : US, EU, JP, CH, RU, XX
proc domain { c } {
   global doms
   set ok 0
   # ---  catch
   catch {
      set d $doms($c)
      set ok 1
   }  rc
   # ---  catch
   if {$ok} { return $d }
   # 
   log [mkErrorCode "!!! Error: Unknown country in array doms: $c"]
   return XX       
}

#   : RnD Unknown Industry Gov.Mil
proc usage { sec aa } {
   global ApplicationAreas Segments Usages
   set ok 0
   catch {
      set xs $Segments($sec)
      set ok 1
   }  rc
   if {!$ok} {
      log [mkErrorCode "!!! Error: Unknown segment in array Segments: $sec"]
      set xs Unknown 
   }
   set ok 0
   catch {
      set xa $ApplicationAreas($aa)
      set ok 1
   }  rc
   if {!$ok} {
      log [mkErrorCode "!!! Error: Unknown ap.area in array ApplicationAreas: $aa"]
      set xa Unknown 
   }
   set ok -1
   catch {
      set ok $Usages($xs)
   }  rc
   if {$ok<0} {
      log [mkErrorCode "!!! Error: Unknown value in array Segments: $xs"]
      set xs Unknown 
   }    
   set ok -1
   catch {
      set ok $Usages($xa)
   }  rc
   if {$ok<0} {
      log [mkErrorCode "!!! Error: Unknown value in array ApplicationAreas: $xa"]
      set xa Unknown 
   }  
   if {$xs ne "Unknown"} {return $xs}
   return $xa
}

