# mkCSVfiles.tcl

# Top500-fields.CSV:         ...
proc top500fields { } {
   global gnames
   log "Top500 Fields"
   openCSV "Top500-fields"
   logCSV "noEds,Field,ed1,ed2,ed3,..."
   foreach {n v} [array get gnames] {
      set s [join $v ","]
      logCSV "[llength $v],$n,$s"
   }
   closeCSV
}

# Top500-field-values.CSV:       
proc top500fieldValues { } {
   global gvals not_collect calcfields
   log "Top500 Fields Values"
   openCSV "Top500-field-values"
   logCSV "noVals,Field,Value"
	 foreach n $not_collect {
         logCSV "n/a,$n,uncollected" 
	 }
	 foreach n $calcfields {
         logCSV "n/a,$n,calculated" 
	 }
   foreach {n v} [array get gvals] {
      set s [llength $v]
      foreach val $v {
         logCSV "$s,$n,$val" 
      }
	 }
   closeCSV
}

# Analize-*.CSV:  CSV-   "name" 
# (, name <- { Cores Proc.Frequency CpT })
#   "Editions,Avg20,Max20,Avg500,Max500"
proc analize1 { name } {
   global gtab max_no

   log "Analize $name"
   openCSV "Analize-$name" 
   logCSV "Editions,Avg20,Max20,Avg500,Max500"         
   for {set no 0} {$no <= $max_no} {incr no} {
      set s20 0
      set m20 0
      set n20 0
      set s500 0
      set m500 0
      set n500 0
      for {set i 1} {$i <= 500} {incr i} {
         set v $gtab($no,$i,$name)
         if {$v eq ""} continue
         if {$i < 21} {
            set n20 [expr $n20 + 1]
            set s20 [expr $s20 + $v]
            if {$m20 < $v} { set m20 $v }
         }
         set n500 [expr $n500 + 1]
         set s500 [expr $s500 + $v]
         if {$m500 < $v} { set m500 $v }
      }
      set edn  [no2edExcel $no]
      set a20  [expr $s20 / $n20]
      set m20  $m20
      set a500 [expr $s500 / $n500] 
      set m500 $m500       
      logCSV "$edn,$a20,$m20,$a500,$m500"         
   }
   closeCSV
}

# Top10Countries.CSV:       Top10
proc analizeTop10 { } {
   global gtab max_no

   log "Country Top10Systems"         
   set cs {}
   openCSV "Top10Countries" 
   logCSV "Country,Top10Systems"         
   for {set no 0} {$no <= $max_no} {incr no} {
      for {set i 1} {$i <= 10} {incr i} {
         set v $gtab($no,$i,Country)
         if { [lsearch $cs $v] < 0 } {
            set t10s($v) 1
            lappend cs $v
         } else {
            set t10s($v) [expr 1 + $t10s($v) ]
         }
      }
   }
   foreach v $cs {
      logCSV "$v,$t10s($v)"
   }
   closeCSV
}

#    no     rmax    i
proc delay { i no rmax } {
   global gtab
   set no1 $no
   while {$no1 >= 0} {
      if {$gtab($no1,$i,RMax) <= $rmax} {
         return [expr double($no - $no1) / 2]
      }
      set no1 [expr $no1 - 1]
   }
   return 100
} 

#   RU  Top500  no (501   RU)
proc ru1 { no } {
   global gtab
   for {set i 1} {$i <= 500} {incr i} {
      if { $gtab($no,$i,Domain) eq "RU" } {
         return $i
      }
   }
   return 501
}

# RussiaDelay.CSV:  ( )   RU  Top1 Top5 Top10
proc rudelay { } {
   global gtab max_no

   log "Russia delay"

   openCSV "RussiaDelay" 
   logCSV "Editions,Delay1,Delay5,Delay10,Rank,Site,Manufacturer,Computer"
   for {set no 0} {$no <= $max_no} {incr no} {
      set iru [ru1 $no]
      set edn [no2edExcel $no]
      if { $iru > 500} {
         logCSV "$edn,100,100,100,501,None,None,None"
         continue
      }
      set rmax $gtab($no,$iru,RMax)
      set d1  [delay  1 $no $rmax]
      set d5  [delay  5 $no $rmax] 
      set d10 [delay 10 $no $rmax]  
      set rank $gtab($no,$iru,Rank)
      set site $gtab($no,$iru,Site)
      set manufacturer $gtab($no,$iru,Manufacturer)      
      set computer $gtab($no,$iru,Computer)
      logCSV "$edn,$d1,$d5,$d10,$rank,$site,$manufacturer,$computer"
   }
   closeCSV
}

# stratification.CSV: ""
proc stratificationCSV { } {
   global gtab max_no
   set no $max_no
   set r0 [expr double($gtab($no,1,RMax))]
   log " 500"
   openCSV "Stratification"
   logCSV "  Top500,Rmax (Tflops),, Top1 ()"
   foreach { i1 i2 } { 1 10 11 20 21 100 101 250 251 500 } {
      set r1 [expr double($gtab($no,$i1,RMax))]
      set r2 [expr double($gtab($no,$i2,RMax))]
      set t1 [expr round($r1/1000)]
      set t2 [expr round($r2/1000)]
      set d  [expr $r1/$r2]
      set d1 [expr round($r0/$r1)]
      set d2 [expr round($r0/$r2)]
      logCSV "Top$i1$i2,$t1$t2,[format "%.1f" $d],$d1$d2"
   }
   closeCSV
}

# CSV- CI-levels-Forecast.CSV:    
# 
proc levelBndrs { nset } {
   set iset {1 5 10 20 21 100 101 250 251 500}
   log "    "
   openCSV "CI-levels-Forecast"
   logCSV0 "Edition"
   foreach i $iset {
      logCSV0 ",$i Up,$i,$i Dn"
   }
   logCSV ""
   foreach no $nset {
      logCSV0 "[no2edExcel $no]"
      foreach i $iset {
         logCSV0 ",[rmax_trendUpC $no $i]"
         logCSV0 ",[rmax_trendC   $no $i]"
         logCSV0 ",[rmax_trendDnC $no $i]"
      }
      logCSV ""
   }
   closeCSV
}

# CSV- Top500-Forecast.CSV:       Top500
proc tops { iset eds } {
   log "      Top500"
   openCSV "Top500-Forecast"
   logCSV0 "Rang,Est"
   foreach ed $eds {
      logCSV0 ",[no2edExcel [ed2no $ed]]"
   }
   logCSV ""
   foreach i $iset {
      logCSV0 "$i,Up"
      foreach ed $eds {
      logCSV0 ",[rmax_trendUpC [ed2no $ed] $i]"
      }
      logCSV ""
      logCSV0 "$i, "
      foreach ed $eds {
      logCSV0 ",[rmax_trendC   [ed2no $ed] $i]"
      }
      logCSV ""      
      logCSV0 "$i,Dn"
      foreach ed $eds {
      logCSV0 ",[rmax_trendDnC [ed2no $ed] $i]"
      }
      logCSV ""
   }
   closeCSV
}
