# plot.tcl

#        

global cW cH cF cF1 plots currMethod stopPressed
set cH 700
set cW 1250
set cF  {Helvetica 12}
set cF1 {Helvetica 9}

set xaw {
}

array set plots {
   ": "    { plotCI                1 }
   ":  : 20/80"  { plot2080              1 }
   ": Usage"                  { plotAnimeUsage        1 }
   ": Segment"                { plotAnimeSegment      1 }
   ": Manufacturer"           { plotAnimeMnfctr       1 }
   ": Interconnect"           { plotAnimeIc           1 }
   ": Chip Technology"        { plotAnimeChip         1 }
   ": "             { plotAnimeStrat        1 }
   ": Top500  "          { plotTrends            0 }
   ": "               { plotPosterStrat       0 }
   ": Usage (SCs)"              { plotPosterUsageSCs    0 }
   ": Usage (RMax)"             { plotPosterUsageRMax   0 }
   ": Segment (SCs)"            { plotPosterSegmentSCs  0 }
   ": Segment (RMax)"           { plotPosterSegmentRMax 0 }
   ": Manufacturer (SCs)"       { plotPosterMnfctrSCs   0 }
   ": Manufacturer (RMax)"      { plotPosterMnfctrRMax  0 }
   ": Interconnect (SCs)"       { plotPosterIcSCs       0 }
   ": Interconnect (RMax)"      { plotPosterIcRMax      0 }
   ": Chip Technology (SCs)"    { plotPosterChipSCs     0 }
   ": Chip Technology (RMax)"   { plotPosterChipRMax    0 }
}

proc plots { } {
   global cW cH cF cF1 plots max_no
   
   set spinM [array names plots]

   catch {destroy .plot}
   toplevel .plot
   wm title .plot "Plot Trends"
   wm iconname .plot "Trends"
   frame .plot.bs -bd 1
   pack .plot.bs -side bottom -fill x

   button  .plot.bs.stop  -text Stop  -command plotStop  -font $cF
   button  .plot.bs.movie -text Movie -command plotMovie -font $cF
   button  .plot.bs.prev  -text "<<"  -command plotPrev  -font $cF
   button  .plot.bs.next  -text ">>"  -command plotNext  -font $cF
   label   .plot.bs.no -text [no2edExcel 0] -font $cF
   
   set len [expr 34 + 3*$max_no]
   scale .plot.bs.scale -orient horizontal -length $len -from 0 -to $max_no \
         -showvalue 0 -tickinterval 0 \
	       -command changedEdition

   button .plot.bs.draw  -text Draw  -command plotDraw -font $cF
   button .plot.bs.close -text Close -command "destroy .plot" -font $cF
   button .plot.bs.exit  -text Exit -command "destroy ." -font $cF
   spinbox .plot.bs.sM -values $spinM -width 30 -font $cF \
           -validate all -vcmd {info exists plots(%P)} 
   pack .plot.bs.movie .plot.bs.prev .plot.bs.next \
        .plot.bs.no .plot.bs.scale .plot.bs.draw .plot.bs.sM \
        .plot.bs.close .plot.bs.exit \
        -side left -expand 1

   canvas .plot.c -width $cW -height $cH -bg white -bd 0
   # -relief raised 
   pack .plot.c -side top -fill x
  .plot.bs.sM set ": "
   update

   #  ,   
   plotDraw
}

proc changedEdition { no } {
   .plot.bs.no configure -text [no2edExcel $no]   
   .plot.bs.scale set $no
}

# ToDo:  Stop  ,   
proc plotStop { } {
   global stopPressed
   set stopPressed 1
   debug 1 
}

proc plotMovie { } {
   global max_no stopPressed
   set n1 {}
   catch { set n1 [limits [.plot.bs.scale get] 0 $max_no] }
   if { [llength $n1] != 1 } {
      set n1 0
   }
   if { $n1 == $max_no } {
      set n1 0
   }
   set stopPressed 0
   for {set no $n1} {$no <= $max_no} {incr no} {
      changedEdition $no
      plotDraw
      after 500
      if { $stopPressed } {
         return
      }
   }
}

proc plotPrev { } {
   global max_no
   set no  [.plot.bs.scale get]
   set no1 $no
   catch { set no1 [limits [expr $no - 1] 0 $max_no] }
   if { $no1 == $no } {return}
   changedEdition $no1
   plotDraw
}

proc plotNext { } {
   global max_no
   set no  [.plot.bs.scale get]
   set no1 $no
   catch { set no1 [limits [expr $no + 1] 0 $max_no] }
   if { $no1 == $no } {return}
   changedEdition $no1
   plotDraw
}

proc setState { bs s } {
   foreach b $bs { .plot.bs.$b configure -state $s } 
}

proc plotDraw { } {
   global cW cH cF cF1 plots
   
   # .plot.bs.sM  
   set m 0
   catch {set m $plots([.plot.bs.sM get])}
   if {[llength $m] != 2} {
      return;
   }
   
   #   
   setState {stop movie prev next no scale sM draw close exit} disabled

   #     
   set p [lindex $m 0]      
   set a [lindex $m 1]

   #  
   .plot.c delete all
   .plot.c create rectangle 0 0 $cW $cH -width 0 -fill ""

   #   
   $p
   
   #   
   if {$a} {
      setState {stop movie prev next no scale sM draw close exit} normal
   } else {
      setState {sM draw close exit} normal
   }
   update
}

#   --  
proc timeScale { no } {
   global cW cH cF1 max_no

   set t [no2edExcel $no]
   .plot.c create text [expr double($cW) / 2] 5 -text $t \
                       -font {Helvetica 24 bold} -anchor n
   set x1 20
   set x2 [expr $cW - 20]
   set y0 [expr 70]
   set y1 [expr $y0 - 10]
   set y2 [expr $y0 + 10]
   .plot.c create line $x1 $y0 $x2 $y0

   
   for {set n 0} {$n <= $max_no} {incr n} {
      set x [linear 0 $x1 $max_no $x2 $n]
      set t [no2edShort $n]
      if {$n % 2} {
         .plot.c create text $x [expr $y2 + 2] -text $t -font $cF1 -anchor n
         .plot.c create line $x $y0 $x $y2
      } else {
         .plot.c create text $x [expr $y1 - 2] -text $t -font $cF1 -anchor s
         .plot.c create line $x $y1 $x $y0
      }
      if {$n == $no} {
         .plot.c create oval [expr $x-7] [expr $y0-7] \
                             [expr $x+7] [expr $y0+7] \
                          -fill red -outline black
      }
   }
}
