# plotTrends.tcl
#     

#  Rax   Y  
proc perf2y { RMax } {
   global cH minP maxP
   set r1 [expr log($minP)]
   set y1 [expr $cH - 40]
   set r2 [expr log($maxP)]
   set y2 10
   set r  [expr log($RMax)]
   return [linear $r1 $y1 $r2 $y2 $r]
}

#  no   X  
proc no2x { n } {
   global max_trend_no cW
   set n1 0
   set x1 70
   set n2 $max_trend_no
   set x2 [expr $cW-20]
   return [linear $n1 $x1 $n2 $x2 $n]
}

#    (RMax)
proc perfLine { RMax Dash } {
   global cF1 cW max_trend_no
   set y [perf2y $RMax]
   set t [perf2txt $RMax]
   .plot.c create text 60 $y -text $t -font $cF1 -anchor e
   if {$Dash} {
      .plot.c create line 65 $y [no2x $max_trend_no] $y -dash ,
   } else {
      .plot.c create line 65 $y [no2x $max_trend_no] $y
   }
}

#    (no)
proc nLine { n } {
   global cF1 maxP minP
   set x  [no2x $n]
   set y1 [perf2y $maxP]
   set y2 [perf2y $minP]
   if {$n % 2} {
      set y2 [expr $y2 + 17]
      .plot.c create text $x [expr $y2 + 2] -text [no2edShort $n] -font $cF1 -anchor n
      .plot.c create line $x $y1 $x $y2 -dash ,
   } else {
      set y2 [expr $y2 + 5]
      .plot.c create text $x [expr $y2 + 2] -text [no2edShort $n] -font $cF1 -anchor n
      .plot.c create line $x $y1 $x $y2
   }
}

#    
proc plotTrends { } {
   global cW cH cF max_trend_no minP maxP ruSCs max_no
   
   set iset {1 10 100 200 300 400 500}
   set tbeg [expr $max_no - 20]

   #   
   .plot.c create rectangle 0 0 $cW $cH -width 0 -fill ""

   #  
   for {set e $minP} {$e < $maxP} {set e [expr $e * 10]} {
     foreach {m d} {1 0 2 1 5 1} {
        perfLine [expr $m*$e] $d
     }
   }
   perfLine $maxP 0
   
   #  
   for {set n 0} {$n <= $max_trend_no} {incr n} {
      nLine $n
   }
   
   #    
   foreach i $iset {
      plotStat  $i
      plotTrand $i $tbeg
   }

   #   
   foreach {no i rmax flag} $ruSCs {
      #   $no   $i  Rmax=$rmax  RU  
      #  ($flag)   
      set x [no2x $no]
      set y [perf2y $rmax] 
      if {$flag} {
         set c red
      } else {
         set c blue
      }
      .plot.c create oval [expr $x-3] [expr $y-3] [expr $x+3] [expr $y+3] \
                          -fill $c -outline black -width 1
   }   

   update
   # .plot.c postscript -file "Results\\top500trends.ps"
}

#   i   Top500
proc plotStat { i } {
   global gtab max_no
   for {set no 0} {$no <= $max_no} {incr no} {
      lappend coords [no2x $no] [perf2y $gtab($no,$i,RMax)] 
   }
   .plot.c create line $coords -fill [i2color $i] -width 2
}

#    i    tbeg
proc plotTrand { i tbeg } {
   global max_trend_no max_no
   set x1   [no2x                 $tbeg]
   set y1   [perf2y [rmax_trend   $tbeg $i]]
   set y1Up [perf2y [rmax_trendUp $tbeg $i]]
   set y1Dn [perf2y [rmax_trendDn $tbeg $i]]

   set x2   [no2x                 $max_no]
   set y2   [perf2y [rmax_trend   $max_no $i]]

   set x3   [no2x                 $max_trend_no]
   set y3   [perf2y [rmax_trend   $max_trend_no $i]]
   set y3Up [perf2y [rmax_trendUp $max_trend_no $i]]
   set y3Dn [perf2y [rmax_trendDn $max_trend_no $i]]
   
   set c    [i2color $i]
   #  
   .plot.c create line $x1 $y1   $x2 $y2   -fill $c -dash - -width 1
   .plot.c create line $x2 $y2   $x3 $y3   -fill $c -dash - -width 3
   #   
   .plot.c create polygon $x1 $y1Up $x3 $y3Up $x3 $y3Dn $x1 $y1Dn \
                          -fill "" -outline $c -dash , -width 1
   # .plot.c create line $x1 $y1Up $x3 $y3Up -fill $c -dash , -width 1
   # .plot.c create line $x1 $y1Dn $x3 $y3Dn -fill $c -dash , -width 1
}
