# trendsTop500.tcl

# . "  Top500.docx"

#  N0, N1, N2, R(i), I0, R_, Q
proc trend_prep { nset iset } {
   global gtab N0 N1 N2 R I0 R_ Q
   set N0 0
   set N1 0
   set N2 0
   set I0 0
   set R_ 0.0
   set Q  0.0
   foreach i $iset {
      set R($i) 0.0
   }
   foreach n $nset {
      set N0 [expr $N0 + 1 ]
      set N1 [expr $N1 + $n ]
      set N2 [expr $N2 + $n * $n ]
      foreach i $iset {
         set R($i) [expr $R($i) + $gtab($n,$i,logRMax)]
         set Q     [expr $Q  + $n*$gtab($n,$i,logRMax)]
      }   
   }
   foreach i $iset {
      set R_ [expr $R_ + $R($i)]
      set I0 [expr $I0 + 1]
   }
   set N0 [expr double($N0)]
   set N1 [expr double($N1)]
   set N2 [expr double($N2)]
   set I0 [expr double($I0)]
}

#  x0 
proc trend_x0 { } {
   global N0 N1 N2 R I0 R_ x0 Q
   set x0 [expr ($N1*$R_ - $N0*$Q) / ($I0 * ($N1*$N1 -  $N0*$N2))]
}

#  xI(i) 
proc trend_xI { iset } {
   global N0 N1 N2 R I0 R_ x0 xI
   foreach i $iset {
      set xI($i) [expr ($R($i) - $N1 * $x0) / $N0]
   }
}

# . "  Top500.docx"
proc trend { nset iset } {
   trend_prep $nset $iset
	 trend_x0   
   set all_i [interval 1 500]
   trend_prep $nset $all_i
   trend_xI   $all_i   
}

# trend2 ( trend)  pc    nset 
proc trend2 { nset pc} {
   global x0 xI gtab xUp xDn
   set drop [expr round((1.0 - $pc) * double([llength $nset]))]
   for {set i 1} {$i <= 500} {incr i} {
      set ds {}
      set xi_ $xI($i)
      foreach no $nset {
         set d1 [expr $gtab($no,$i,logRMax) - $x0 * $no]
         set d2 [expr abs($d1 - $xi_)]
         lappend ds [list $d1 $d2]
      }
      set ds [lsort -decreasing -real -index 1 $ds]
      set ds [lrange $ds $drop end]
      set ds [lsort -decreasing -real -index 0 $ds]
      set xUp($i) [lindex $ds 0   0]
      set xDn($i) [lindex $ds end 0]
   }
}

#   rmax  no i
proc rmax_trend { no i } {
   global x0 xI
   return [expr exp($x0*$no + $xI($i))]
}

#   rmax  no i
proc rmax_trendUp { no i } {
   global x0 xUp
   return [expr exp($x0*$no + $xUp($i))]
}

#   rmax  no i
proc rmax_trendDn { no i } {
   global x0 xDn
   return [expr exp($x0*$no + $xDn($i))]
}

#   rmax  no i ( )
proc rmax_trendC { no i } {
   global x0 xI gtab max_no
   if {$no <= $max_no} { return $gtab($no,$i,RMax) }
   return [max [expr exp($x0*$no + $xI($i))] $gtab($max_no,$i,RMax) ]
}

#   rmax  no i ( )
proc rmax_trendUpC { no i } {
   global x0 xUp gtab max_no
   if {$no <= $max_no} { return $gtab($no,$i,RMax) }
   return [max [expr exp($x0*$no + $xUp($i))] $gtab($max_no,$i,RMax) ]
}

#   rmax  no i ( )
proc rmax_trendDnC { no i } {
   global x0 xDn gtab max_no
   if {$no <= $max_no} { return $gtab($no,$i,RMax) }
   return [max [expr exp($x0*$no + $xDn($i))] $gtab($max_no,$i,RMax) ]
}

#   exp(2*x0)  ---    1 
# CSV- TrendFunc.CSV
proc trendfunc { } {
   global max_no x0
   log "  exp(2*x0)  ---    1 "
   openCSV "TrendFunc"
   
   set all_i [interval 1 500]
   set sel_i {1 10 20 100 200 300 400 500}
   set d0 [expr int($max_no/2)]
   set d0 [expr ( $d0 < 14 ? 14 : $d0 ) ]
   set d0 [expr ( $d0 > $max_no ? $max_no : $d0 ) ]
	 set maxDiff 0.0
	 set minDiff 0.0   
   logCSV0 "ed"
   for {set d $d0} {$d <= $max_no } {incr d} {
      logCSV0 ",s($d),f($d)"
   }
   logCSV ""   
   for {set b $d0} {$b <= $max_no} {incr b} {
      logCSV0 [no2edExcel $b]
      for {set d $d0} {$d <= $max_no } {incr d} {  
        if {$b < $d} {
           logCSV0 ",,"
           continue
        }
        set set_n [interval [expr $b - $d] $b] 
        trend_prep $set_n $sel_i
	      trend_x0
	      set x $x0
	      logCSV0 ",[expr exp(2*$x0)]"
        trend_prep $set_n $all_i
	      trend_x0
	      logCSV0 ",[expr exp(2*$x0)]"
	      set x  [expr $x - $x0]
	      if {$maxDiff < $x} {set maxDiff $x}
	      if {$minDiff > $x} {set minDiff $x}
     }
     logCSV ""
   }
   logCSV "  select-full:,$minDiff,..,$maxDiff" 
   closeCSV
}
